//Power and SWR meter with optional Magnetic Loop controller
//
//Author: Kevin Ayriss G8MXV
//
//Designed for and Arduino Nano
//
//Version History
//Version 1.0:  First version. Migrated from board 1.0 11/6/19
//*****For version 1.1 PCBs ONLY***** 
//Version 2.0:  Added Relay 1 and 2 controls as well as end point detect set up
//              Added Dual capacitor tuning and set up control
//Version 2.1   Changed to using Timer 0 Compare A interupt for the 1 ms Interupt
//Version 2.2   Restructured navigation between dispaly screens
//Version 2.3   Extended the settings menu to allow the band capacitor to be manually adjusted.
//Version 2.4   Combined the band capacitor tuning with the main tuning algorithm (11/9/19)
//Version 2.5   Provided option to opperate a single stepper with band switching relays and no end point detect
String version = "Version 2.5";

//**********************************
#include <LiquidCrystal.h>  //library to drive the LCD display
#include <EEPROM.h>         //Library to access the EP prom to store settings when powered down


// set pin assignment:
//*****For version 1.1 PCBs ONLY*****
const unsigned int encoder_clock = 9;         //pulled up, low = operated  encoder clock
const unsigned int encoder_data = 10;          //pulled up, low = operated  encoder data
const unsigned int encoder_switch_pin = 11;    //pulled up, low = operated  encoder switch

const unsigned int LCD_D4 = 4;                //LCD control leads
const unsigned int LCD_D5 = 5;
const unsigned int LCD_D6 = 6;
const unsigned int LCD_D7 = 7;
const unsigned int LCD_E = 2;
const unsigned int LCD_RS = 3;
const unsigned int back_light = 8;  
              
const unsigned int Stepper_1 = A3;            //stepper motor control leads
const unsigned int Stepper_2 = A2; 
const unsigned int Stepper_3 = A1; 
const unsigned int Stepper_4 = A0;

const unsigned int Function_select= 12;       //0= meter alone 1 = with mag loop 

const unsigned int Relay_1 =13;               //relay leads
const unsigned int Relay_2 =A4;

const unsigned int vforward =A6;              //A to D Meter readings
const unsigned int vreverse =A7;

const unsigned int end_point_detect =A5;


//define Interupt Service Routine (ISR) flags 

volatile boolean long_press = false;
volatile boolean double_press = false;
volatile boolean single_press = false;
volatile boolean encoder_changed_state = false;
volatile boolean display_timer_expired = false;

volatile boolean encoder_clock_last_stable_state = false;
volatile boolean encoder_clock_debounce_flag = false;
volatile boolean encoder_direction = LOW;
volatile boolean encoder_switch_button_held_flag = LOW;
volatile boolean encoder_switch_half_sec_timer_flag =LOW;
volatile boolean encoder_switch_last_stable_state=LOW;
volatile boolean encoder_switch_debounce_flag=LOW;


//define Interupt Service Routine (ISR) timers 
volatile unsigned int encoder_clock_timer=0;
volatile unsigned int encoder_switch_timer=0;
volatile unsigned int encoder_switch_half_sec_timer =0;
volatile unsigned int encoder_switch_two_second_timer=0;
volatile unsigned int PK_Power_timer=0;
volatile unsigned int stepper_on_timer=0;
volatile unsigned int step_interval_timer=0;
volatile unsigned int wait_for_power_timer=0;
volatile unsigned int wait_for_power_stable_timer=0;
volatile unsigned int display_timer=0;


//define Interupt Service Routine (ISR) timer values
const unsigned int enc_debounce_time =2;  //2 ms 
const unsigned int enc_switch_debounce_time =2;  //2 ms

//Set up LCD display
//
// syntax:- LiquidCrystal(rs, enable, d4, d5, d6, d7) 

LiquidCrystal lcd(LCD_RS, LCD_E, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

//general flags
boolean Function = LOW;      //LOW= meter alone HIGH = with mag loop

boolean power_present=LOW;  //HIGH = RF power is present
boolean power_warning=LOW;  //HIGH = waring power too high
boolean power_reversed=LOW; //HIGH= inputs are reversed
boolean power_stable=LOW;   //HIGH= RF power stable
boolean power_stable_timer_flag=LOW; //LOW = RF power stable time has not been set, HIGH = it has been set and is running
 
boolean stepper_powered =LOW; //HIGH = stepper is turned on
boolean stepper_speed=HIGH;   //HIGH = fast LOW = slow
boolean stepper_direction=LOW; //LOW = clockwise HIGH= Anticlockwise

boolean tuning_in_progress=LOW; //HIGH= tuning is in progress
boolean tuning_reverse_check=LOW; //LOW = stepper reverse tuning has not been used  HIGH= it has been used to ensure stepper direction is only reversed once.
boolean band_cap_setup=LOW;     //HIGH = band capacitor is being tuned

boolean end_point_warning=LOW;  //HIGH =endpoint detector input voltage to high
boolean end_point_detector_one = LOW; //HIGH = end point dectector 1 is active
boolean end_point_detector_two = LOW; //HIGH = end point dectector 2 is active
 


//Display mode values, used to determine which screen is being displayed at a particular time
const unsigned int main_screen =0;
const unsigned int settings_menu_1 =1;
const unsigned int settings_band =2;
const unsigned int settings_scale =3;
const unsigned int settings_backlight =4;
const unsigned int inputs_mode =5;
const unsigned int configuration_mode =6;
const unsigned int default_restored =7;
const unsigned int show_band_cap_values =8;
const unsigned int settings_menu_2 =9;
const unsigned int band_cap_manual_adj=10;
const unsigned int band_capacitor_tuning_screen=11;
const unsigned int band_cap_to_start_epd_error=12;
const unsigned int main_cap_to_start_epd_error=13;
const unsigned int settings_controller_type=14;



// define constants 
const unsigned int minimum_power = 26; //A-D value that equates to approx 100mW
const unsigned int maximum_power = 1000; //A-D value that equates yo approx 138w   
const float V_ref=4.8; //Referance voltage 4.8 volts
const unsigned int stepper_on_time=1000; // when not stepping turn the stepper current off after  1 second
const unsigned int step_interval_time=10; //10ms between steps
const unsigned int step_interval_time_slow=50; //50ms between steps slow speed
const unsigned int tune_message_time=5000; //persist tune messages TUNE, NOK NoPW and OK  on screen for 5 seconds
const unsigned int deslack_count=15; // number of steps required to take up slack in stepper gear train when changing direction

//step counter - each step = 11.25 degrees and there is a 64:1 gear box following that so there are 360/11.25 *64 = 2048
//steps therefore 180 degrees needs a count of 2048 / 2= 1024 
const unsigned int max_step_count=2048;  //Assuming the capacitor could be in any position allow it to travel full circle to find the end point 
const unsigned int mid_c_position=256; //max c is 90 degrees from min C so mid C = 45 deg. ie 2048/8 = 256 steps 
const unsigned int wait_for_power_time= 6000; //when setting up band capacitor allow six seconds for the tx to be applied, three seconds during please turn on power message and 3 seconds after that
const unsigned int wait_for_power_stable_time=1000; //wait for one second to allow power to stablise before starting to tune

//controller types
#define relay_type 0
#define dual_stepper 1

//endpoint detection assumes 10mA is passed into the detector for EP 1
//20mA for EP 2 and if both are opperated 30mA
// current is passed through a 47 ohm resistor 
//referance voltage is 4.8 volts
//EP 1 A2D voltage = 0.01*47*1023/4.8= 100 
//allowing for a 10% tollarance
const unsigned int epd_1_min = 90;
const unsigned int epd_1_max =110;
const unsigned int epd_2_min = 180;
const unsigned int epd_2_max =220;
const unsigned int epd_1and2_min= 270;
const unsigned int epd_1and2_max= 330;
const unsigned int epd_max = 500;  // this is equivlant to applying 7.3 volts to the epd terminal 

//Band and Main tuning capacitor directions
const boolean band_cap_minimum_direction=HIGH; // direction to drive band stepper for minumum C ie: anticlockwise
const boolean band_cap_increase_C_direction=LOW; // direction to drive band stepper to increase C ie: clockwise
const boolean main_tuning_cap_minimum_direction=HIGH; // direction to drive main tuning stepper for minumum C ie anticlockwise 
const boolean main_tuning_cap_increase_C_direction=LOW; // direction to drive main tuning stepper to increase C ie: clockwise
 

//variables
int Vfor=0;     //store for the current rolling average value for the forward voltage
int Vrev=0;     //store for the current rolling average value of the reverse voltage
float Power=0;  //used to store the calculated value of the RF power
float SWR =0;   //used to store the calculated value of the SWR
float PK_Power=0; //store for the peak power reading
int bar_value=0;  //used in the bargraph routine to store the current bar value
int bar_peak=0; //used in the bar graph routine to store the peak bar value.

float starting_swr=0; //used in the tune routine to store the starting value of the SWR to see if when tuning the SWR is getting better or worse.

int Vepd=0; //used to store the end point dectector value when it is read


byte display_mode= main_screen;  //used to store the display screen number 
int settings_cursor=0;  //used to determine the current cursor location
int cursor_max; // this is the max count that settings_cursor can get to before it is reset to zero. This can hold different values depending on the number of items to select on a screen.
int band=0; //store for the band in use, 0=40M, 1=30M, 2=20M
int scale=0;  //store for the scale setting 0=0-5W, 1= 0-20W and 2= 0-125W
int Backlight=1; //backlight store, 0=backlight off, 1=backlight on
int controller_type=0; // 0= Relay band switched type, 1= dual stepper band and tune

int step_counter=0; //used to count the number of steps the stepper has made 
int stepper_phase=0; // used in the stepper routine to determine the next output patten to make the stepper move in the appropriate direction.



// Cursor positioning arrays for the various screens
unsigned int settings_mag_loop_menu_cursor_array[12]={0,1,0,2,0,3,8,1,8,2,8,3};
unsigned int settings_mag_loop_menu_cursor_array_2[8]={0,1,0,2,0,3,10,3};
unsigned int settings_power_meter_menu_cursor_array[8]={0,1,0,2,0,3,8,1};
unsigned int settings_scale_cursor_array[6]={0,1,0,2,0,3};
unsigned int settings_band_cursor_array[6]={0,1,0,2,0,3};
unsigned int settings_backlight_cursor_array[4]={0,1,0,2};
unsigned int configuration_mode_cursor_array[6]={0,1,0,2,0,3};
unsigned int settings_controller_type_cursor_array[4]={0,1,0,2};

//band capacitor position settings array
int band_cap_settings[3] ={-1,-1,-1}; // band capacitor setting values, defaulted to not set up =-1

//rolling average storage arrays for V forward and V reverse
unsigned int vforward_average_array[20];
unsigned int vreverse_average_array[20];
int rolling_average_index=0;


//bargraph symbol setup 
//code_0 two dots ..
byte code_0[8] = {
B00000000,
B00000000,
B00000000,
B00010010,
B00000000,
B00000000,
B00000000,
B00000000,
};

//code_1 power bar dot |.
byte code_1[8] = {
B00000000,
B00010000,
B00010000,
B00010010,
B00010000,
B00010000,
B00000000,
B00000000,
};

//code_2 2 power bars ||
byte code_2[8] = {
B00000000,
B00010010,
B00010010,
B00010010,
B00010010,
B00010010,
B00000000,
B00000000,
};

//code_3 peak bar dot 
byte code_3[8] = { 
B00011000,
B00011000,
B00011000,
B00011010,
B00011000,
B00011000,
B00011000,
B00000000,
};

//code_4 power bar peak bar 
byte code_4[8] = { 
B00000011,
B00010011,
B00010011,
B00010011,
B00010011,
B00010011,
B00000011,
B00000000,
};

//code_5 dot peak bar
byte code_5[8] = {  
B00000011,
B00000011,
B00000011,
B00010011,
B00000011,
B00000011,
B00000011,
B00000000,
};
 
void setup() {
//Serial.begin(9600);  //not used but left in case there is a need for it in the future for example duing debugging
 
//set up port pins
pinMode(encoder_clock,INPUT_PULLUP);
pinMode(encoder_data,INPUT_PULLUP);
pinMode(encoder_switch_pin,INPUT_PULLUP);

pinMode(Function_select,INPUT_PULLUP);

pinMode(LCD_D4,OUTPUT);
pinMode(LCD_D5,OUTPUT);
pinMode(LCD_D6,OUTPUT);
pinMode(LCD_D7,OUTPUT);
pinMode(LCD_E,OUTPUT);
pinMode(LCD_RS,OUTPUT);
pinMode(back_light,OUTPUT);

pinMode(Stepper_1,OUTPUT);
pinMode(Stepper_2,OUTPUT);
pinMode(Stepper_3,OUTPUT);
pinMode(Stepper_4,OUTPUT);

pinMode(Relay_1,OUTPUT);
pinMode(Relay_2,OUTPUT);

pinMode(vforward,INPUT);
pinMode(vreverse,INPUT);

pinMode(end_point_detect,INPUT);

//Set up external referance voltage

analogReference(EXTERNAL);


// Set the LCD address to a 16 chars and 4 line display
//lcd.begin(cols, rows)
lcd.begin(16, 4);  //set up LCD with 16 coloums, 4 rows

//Set up bar graph characters
lcd.createChar(0, code_0);
lcd.createChar(1, code_1);
lcd.createChar(2, code_2);
lcd.createChar(3, code_3);
lcd.createChar(4, code_4);
lcd.createChar(5, code_5);


//Turn off the stepper motor output
stepper_phase=0;
stepper_off();
stepper_speed=HIGH;
stepper_direction=LOW;  //set initial direction

//Turn off both relay outputs
digitalWrite(Relay_1, LOW); 
digitalWrite(Relay_2, LOW); 

//Read Function_select setting
Function= digitalRead(Function_select);     // read pin and store 0= meter alone 1 = with mag loop 


//read eeprom for band, scale, back light and band capacitor setting
int value = EEPROM.read(0); 
value = value ^ EEPROM.read(1); //exclusive or the two value to see if they are 0xff or 255
if (value !=255) set_ee_to_default_values();  //if not restore ee defaults 

band= EEPROM.read(2);
scale=EEPROM.read(3);
Backlight=EEPROM.read(4);

//get band capactor stepper positions 
//note these aree intergers so two EEPROM bytes are read per value 
EEPROM.get(5, band_cap_settings[0]);
EEPROM.get(7, band_cap_settings[1]);
EEPROM.get(9, band_cap_settings[2]);
controller_type=EEPROM.read(11);


// Operate blacklight 
if (Backlight==1)
  {
    digitalWrite(back_light, HIGH);
  }
  else
  {
    digitalWrite(back_light, LOW); 
  }

//if in dual stepper mode 
//ensure the stepper is controlling the main tuning capacitor
if ( controller_type==dual_stepper) point_stepper_to_main_tuning_capacitor();


//clear rolling average arrays
clear_average_arrays();


//set up start screen 
//Set up display and send version number.
// (note: line counting begins with 0)

lcd.clear();  //clear display

lcd.setCursor(0, 0);
if (Function==HIGH) //see which mode the meter is in 
  {
    lcd.print("Mag loop control");
  }
else
  {
  lcd.print("Power Meter");
  }

lcd.setCursor(0, 1);
lcd.print("Power Range 125W");
lcd.setCursor(0, 2);
lcd.print("(C) G8MXV 2019");
lcd.setCursor(0, 3);
lcd.print(version); 


//clear rotary encoder, button and display timerflags
clear_flags (); 

//set up timer interupt
// Timer0 is already used for millis() - will interrupt 
// in the middle and call the "Compare A" function below
OCR0A = 0x80;
TIMSK0 |= _BV(OCIE0A);

delay (3000); //delay before setting up band capacitor

//Test code to allow mode to be changed during testing
if (long_press==HIGH)
  {
   long_press=LOW;
   if (Function==HIGH)
    {
      Function=LOW; 
    }
   else
   {
    Function=HIGH;
   }
  }
// end of test code

if (Function==HIGH && controller_type==dual_stepper)
  {
  //In mag loop mode and dual stepper mode, set the band capacitor to the store value, if there is one.
  if(band_cap_settings[band]!=-1) //if the band cap value is not equal to -1 ie not set 
    {
    lcd.clear();  
    lcd.setCursor(0, 0);
    lcd.print("Setting Band");
    lcd.setCursor(0, 1);
    lcd.print("Capacitor to"); 
    lcd.setCursor(0, 2);
    lcd.print("stored value.");
    lcd.setCursor(0, 3);
    lcd.print("Please wait..");
    set_band_cap_to_value();
    }
  }

if (Function==HIGH && controller_type==relay_type)
  {
   set_up_band_relays(); 
  }
  

//now set up main screen

//set up main screen
set_up_main_screen();

}

void loop() 
{
//main program loop

while( long_press==LOW && double_press==LOW && single_press==LOW && encoder_changed_state==LOW && display_timer_expired==LOW)
  {
  // see if stepper needs turning off
  if (stepper_powered==HIGH)
    {
    if(stepper_on_timer==0) stepper_off(); 
    }
    
  // Read endpont detctors  
  read_end_point_detectors();

  if (end_point_warning==HIGH) 
    {
    display_endpoint_voltage_too_high_warning();  
    }
    
  //Read forward and reverse inputs
  get_bridge_inputs();
  if (power_warning==HIGH) display_power_too_high_warning(); //input power level too high display warning till issue resolved
  if (power_reversed==HIGH) display_inputs_reversed_warning(); //meter inputs reversed display warning till issue resolved

  calculate_power_and_SWR();
  
  if ( display_mode==main_screen)
    {
     //in normal mode
     // display power and SWR
     display_power_and_SWR();
     display_bargraph();
    }
  if (display_mode==inputs_mode) display_values();

  if (display_mode==band_cap_manual_adj)
    {
     // In band capacitor manual adjustment mode
     //band cap adj is shown instead of bar graph
     display_power_and_SWR(); 
    }
  
  if (tuning_in_progress==HIGH) tune();  
  }

//Service the button, encoder actions and display timer actions

if ( long_press==HIGH) long_press_service();

if (double_press==HIGH) double_press_service(); 

if (single_press==HIGH) single_press_service();

if (encoder_changed_state==HIGH) encoder_service();

if (display_timer_expired==HIGH) display_timer_service();

}

//******************************************
// Routines
//******************************************




//*********************************
//
//Main service routines
//
//*********************************

void single_press_service()
{
//This service is called when the encoder button on the mag loop 
//controler or the push button on the power meter has been single pressed.

single_press=LOW; //clear flag

if (Function==HIGH)
  {
    //in mag loop mode
   if (tuning_in_progress==LOW) //ignore if tuning up is in progress
    {
    switch (display_mode)
      {
      case main_screen:
        //single press was to start tuning.
        tune(); //activate tune if not already activated 
        break;
      case settings_menu_1:
        // single press was to select an option on the settings menu.
        switch (settings_cursor) 
          {
          case 0:
            //set up the band selector
            set_up_band_screen();
            break;
          case 1:
            //set up the scale selector
            set_up_scale_screen(); 
            break;
          case 2:
            //allow backlight to be turned on or off
            set_up_backlight_screen();
            break;
          case 3:
            //go to controller type selector
            set_up_controller_type_screen();
            break;
          case 4:
            //go to settings screen 2
            set_up_mag_loop_settings_screen_2 ();
            break;
          case 5:
            //Exit
            set_up_main_screen();
            break;   
          default:
            set_up_main_screen();
            break;
          }
        break;
        
      case settings_band:
        // single press to select band 
        //set band to cursor setting
        band = settings_cursor;
        update_ee();

        if (controller_type==dual_stepper)
          {
          //controller in dual stepper mode
           if(band_cap_settings[band]==-1)
            {
            //Band cap not set up so can't set it to a value
            //return to main screen          
            set_up_main_screen(); 
            }
           else
            {
            lcd.clear();  
            lcd.setCursor(0, 0);
            lcd.print("Setting Band");
            lcd.setCursor(0, 1);
            lcd.print("Capacitor to"); 
            lcd.setCursor(0, 2);
            lcd.print("stored value.");
            lcd.setCursor(0, 3);
            lcd.print("Please wait..");
            set_band_cap_to_value();  

            //clear flags in case anything was pressed during the set up
            clear_flags (); 
            //return to main screen          
            set_up_main_screen(); 
            }
          }
         else
          {
          //controller in band relay type
          //set up the band relays
          set_up_band_relays();
          //return to main screen
          set_up_main_screen(); 
          }
        break;
        
      case settings_scale:
        // single press was to select scale 
        //set scale to cursor setting
        scale = settings_cursor;
        update_ee();
        //return to main screen
        set_up_main_screen();
        break;
        
      case settings_backlight:
        // single press to select backlight 
        //set back_light = settings_cursor;
        Backlight=settings_cursor;
        if (Backlight==1)
         {
         digitalWrite(back_light, HIGH);  
         }
        else
         {
         digitalWrite(back_light, LOW);
         }
        update_ee();
        //return to main screen
        set_up_main_screen();      
        break;
        
      case settings_menu_2:
        switch (settings_cursor) 
          {
          case 0:
            //allow band capacitor to be manually adjusted if in dual stepper mode
            if (controller_type==dual_stepper)
              {
                set_up_band_capactor_manual_adj_screen();
                point_stepper_to_band_tuning_capacitor();
              }
            //if not in dual stepper ignore the press
            break;
          case 1:
            //set up the inputs screen
            set_up_inputs_display_screen();
            break;
          case 2:
            //restore defaults
            set_ee_to_default_values();
            set_up_defaults_restored_screen();
            break;
          case 3:
            //Exit
            set_up_main_screen();
            break;
          }
          break; 
          
      case inputs_mode:
        // single press to exit inputs mode 
        set_up_main_screen();
        break; 
        
      case configuration_mode:
        //single press was to select an option in the configuration menu
        switch (settings_cursor) 
          {
          case 0:
            //set main tuning capacitor to start
            lcd.clear();  
            lcd.setCursor(0, 1);
            lcd.print("Please wait");
            if (set_tuning_cap_to_start()==0)
              {
              lcd.clear();  
              lcd.setCursor(0, 1);
              lcd.print("Main Capacitor");
              lcd.setCursor(0, 2);
              lcd.print("set to start"); 
              delay (3000);
              }
            //clear flags in case anything was pressed during the set up
            clear_flags (); 
            //return to main screen
            set_up_main_screen();                 
            break;
          case 1:
            //set band capacitor to start
            lcd.clear();  
            lcd.setCursor(0, 1);
            lcd.print("Please wait");
            if (set_band_cap_to_start()==0)
              {
              lcd.clear();  
              lcd.setCursor(0, 1);
              lcd.print("Band Capacitor");
              lcd.setCursor(0, 2);
              lcd.print("set to start"); 
              delay (3000);
              }
            point_stepper_to_main_tuning_capacitor();
            //clear flags in case anything was pressed during the set up
            clear_flags (); 
            //return to main screen
            set_up_main_screen();
            break;
          case 2:
            //Display the band capacitor values
            display_band_capacitor_values ();
            break;
          }
        break;
        
      case band_cap_manual_adj:
        //single press to exit band capacitor manual adj mode.
        point_stepper_to_main_tuning_capacitor();
        //return to main screen
        set_up_main_screen();
        break;
        
      case settings_controller_type:
        //single press to select controller type
        //set controller_type to cursor setting;
        controller_type=settings_cursor;
        update_ee();
        
        //set up the band relays or position the band capacitor as appropriate.
        if (controller_type==dual_stepper)
          { 
          //controller in dual stepper mode
          point_stepper_to_main_tuning_capacitor(); 
          if(band_cap_settings[band]==-1)
            {
            //Band cap not set up so can't set it to a value
            //return to main screen          
            set_up_main_screen(); 
            }
          else
            {
            lcd.clear();  
            lcd.setCursor(0, 0);
            lcd.print("Setting Band");
            lcd.setCursor(0, 1);
            lcd.print("Capacitor to"); 
            lcd.setCursor(0, 2);
            lcd.print("stored value.");
            lcd.setCursor(0, 3);
            lcd.print("Please wait..");
            set_band_cap_to_value();  

            //clear flags in case anything was pressed during the set up
            clear_flags (); 

            //return to main screen
            set_up_main_screen(); 
            }
          }
        else
          {
          //controller in band relay type
          //set up the band relays
          set_up_band_relays();
          //return to main screen
          set_up_main_screen(); 
          }     
        break;
      }
    }
  }
else
  {
  //Power meter functions

  switch (display_mode)
    {
    case main_screen:
      //single press was to change scale
      scale=scale+1;
      if (scale==3) scale=0; //role back to 0-5W  
      lcd.setCursor(10, 2);
      if (scale==0) lcd.print("0-5W  ");
      if (scale==1) lcd.print("0-20W ");
      if (scale==2) lcd.print("0-125W");
      update_ee();   
      break;
    case settings_menu_1:
      //single press was to increment the cursor in the settings menu
      //increment cursor position
      increment_setting_pointer( settings_power_meter_menu_cursor_array);
      break;
    case inputs_mode:
      // single press to exit inputs mode 
      set_up_main_screen();
      break;
    case settings_backlight:
      //single press was to increment the cursor in the backlight menu
      //increment cursor position
      increment_setting_pointer( settings_backlight_cursor_array);
      break;
    }
  }
return;
}

//******************

void double_press_service()
{
//This service is called when the encoder button on the mag loop 
//controler or the push button on the power meter has been double pressed.
 
double_press=LOW; //clear flag

if (Function==HIGH)
  {
  //in mag loop mode
  if (tuning_in_progress==LOW && band_cap_setup==LOW) //ignore if tuning or band cap set up is in progress
    {
    switch (display_mode)
      {
      case main_screen:
        //double press was to get to settings screen
        set_up_mag_loop_settings_screen ();
        break;
      } 
    }
  }
else
  {
   //power meter mode
   // no service currently defined in power meter mode
  }
return;
}
//******************




void encoder_service()
{
encoder_changed_state=LOW;  //Clear flag

if (Function==HIGH)
  {
  // Magloop controller functions
  switch (display_mode)
      {
      case main_screen:
      case inputs_mode:
        //in main screen and inputs mode encoder is used to adjust the main tuning capacitor
        //see if tuning or band cap setting is in progress if so dont change the stepper
        if (tuning_in_progress==LOW && band_cap_setup==LOW)
          {
           if (step_interval_timer==0)
             {
             if (encoder_direction==LOW)
               {
               if (stepper_direction==HIGH)
                 {
                 stepper_direction=LOW;
                 stepper_deslack();
                 }
               }
             else
               {
               if (stepper_direction==LOW)
                 {
                 stepper_direction=HIGH;
                 stepper_deslack();                  
                 }
               }
             opperate_stepper();
             }
          }  
        break;
        


        
      case band_cap_manual_adj:
        //in band_cap_manual_adj mode encoder is used to adjust the band tuning capacitor
        //see if tuning or band cap setting is in progress if so dont change the stepper
        if (tuning_in_progress==LOW && band_cap_setup==LOW)
          {
           if (step_interval_timer==0)
             {
             if (encoder_direction==LOW)
               {
               if (stepper_direction==HIGH)
                 {
                 stepper_direction=LOW;
                 stepper_deslack();
                 }
               }
             else
               {
               if (stepper_direction==LOW)
                 {
                 stepper_direction=HIGH;
                 stepper_deslack();                  
                 }
               }
             opperate_stepper();
             }
          }  
        break;
      case settings_menu_1:
        if (encoder_direction==LOW)
          {
          //clock wise increment cursor position
          increment_setting_pointer( settings_mag_loop_menu_cursor_array);
          }
        else
          {
          //anticlock wise deccrement cursor position
          decrement_setting_pointer( settings_mag_loop_menu_cursor_array);
          }
        break;
        
      case settings_band:
         if (encoder_direction==LOW)
           {
           //clock wise increment cursor position
           increment_setting_pointer( settings_band_cursor_array);
           }
         else
           {
           //anticlock wise deccrement cursor position
           decrement_setting_pointer( settings_band_cursor_array);
           }  
        break;
        
      case settings_scale:
        if (encoder_direction==LOW)
          {
          //clock wise increment cursor position
          increment_setting_pointer( settings_scale_cursor_array);
          }
        else
          {
          //anticlock wise deccrement cursor position
          decrement_setting_pointer( settings_scale_cursor_array);
          }
        break;
        
      case settings_backlight:
        if (encoder_direction==LOW)
          {
          //clock wise increment cursor position
          increment_setting_pointer( settings_backlight_cursor_array);
          }
        else
          {
          //anticlock wise deccrement cursor position
          decrement_setting_pointer( settings_backlight_cursor_array);
          }
        break;
        
      case configuration_mode:
        if (encoder_direction==LOW)
          {
          //clock wise increment cursor position
          increment_setting_pointer( configuration_mode_cursor_array);
          }
        else
          {
          //anticlock wise deccrement cursor position
          decrement_setting_pointer( configuration_mode_cursor_array);
          }
        break;
        
      case settings_menu_2:
        if (encoder_direction==LOW)
          {
          //clock wise increment cursor position
          increment_setting_pointer( settings_mag_loop_menu_cursor_array_2);
          }
        else
          {
          //anticlock wise deccrement cursor position
          decrement_setting_pointer( settings_mag_loop_menu_cursor_array_2);
          }
        break; 

       case settings_controller_type:
        if (encoder_direction==LOW)
          {
          //clock wise increment cursor position
          increment_setting_pointer( settings_controller_type_cursor_array);
          }
        else
          {
          //anticlock wise deccrement cursor position
          decrement_setting_pointer( settings_controller_type_cursor_array);
          }
        break; 
       
      }
  }
 else
  {
  //Rotary encoder not supported in power meter mode     
  }
 return;
 }


//******************

void long_press_service()
{
//This service is called when the encoder button on the mag loop 
//controler or the push button on the power meter has been pressed for more that 2 seconds.
long_press=LOW; //Clear flag

if (Function==HIGH)
  {
  // mag loop mode
  if (tuning_in_progress==LOW && band_cap_setup==LOW) //ignore if tuning or band cap set up is in progress
    {
    switch (display_mode)
      { 
      case main_screen:
        //in main screen if conroller is in dual stepper mode long press was to set up configuration screen
        //if not the screen is notr available
        if (controller_type==dual_stepper) set_up_configuration_mode_screen ();
        break;
      case settings_band:
        //long press when in settings band indicated the band point at by the cursor need to be reset
        //clear band cap setting
        band_cap_settings[settings_cursor]=-1;
        //re display the band screen. Note cursor will be set to the current band not necessarly the band that was reset
        set_up_band_screen();
        break;
      }  
    }
  }
else
  {
  //power meter mode
  switch (display_mode)
    { 
    case main_screen:
      //long press in main screen is to request the settings screen to be displayed
      set_up_power_meter_settings_screen ();
      break; 
    case settings_menu_1:
      //long press was to select item in settings menu
      switch (settings_cursor) 
        {
        case 0:
          //allow backlight to be turned on or off
          set_up_backlight_screen();
          break;
        case 1:
          //set up the inputs screen
          set_up_inputs_display_screen();
          break;
       case 2:
          //restore defaults
          set_ee_to_default_values();
          set_up_defaults_restored_screen();
          break;
        case 3:
          //Exit
          set_up_main_screen();
          break;   
        default:
          set_up_main_screen();
          break;
        }
      break;
    case settings_backlight:
      //long press was to select the back light setting
      //set back_light = settings_cursor;
      Backlight=settings_cursor;
      if (Backlight==1)
        {
        digitalWrite(back_light, HIGH);  
        }
      else
        {
        digitalWrite(back_light, LOW);
        }
      update_ee();
      //return to main screen
      set_up_main_screen();
      break;  
    }
  }
return;  
} 
//******************


void display_timer_service()
{
// This routine allows displays to change state when the display timer expires
display_timer_expired=LOW; //Clear flag

switch (display_mode)
  {
  case default_restored:
    //return to main screen
    set_up_main_screen();
    break;

  case show_band_cap_values:
      //return to main screen
      set_up_main_screen();
      break;
   case main_screen:
      //used to clear tune messages 
      lcd.setCursor(12, 3);
      lcd.print("    ");
      break;
    case band_capacitor_tuning_screen: 
      //band capacitor tuning 
      //turn TX on message expired 
      set_up_main_screen();
      // show tune message
      lcd.setCursor(12, 3);
      lcd.print("Tune");
      break;
    case band_cap_to_start_epd_error:
      //band cap epd error
      set_up_main_screen();
      break;
    case main_cap_to_start_epd_error:
      //band cap epd error
      set_up_main_screen();
      break;        
  }

return;
}
//******************

//**************************************
//
//Display routines
//
//**************************************


void set_up_main_screen()
{
//set up main screen

//set display mode to main screen
display_mode= main_screen;

lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Power");
lcd.setCursor(7, 0);
lcd.print("Peak");
lcd.setCursor(12, 0);
lcd.print("VSWR");
lcd.setCursor(10, 2);
if (scale==0) lcd.print("0-5W  ");
if (scale==1) lcd.print("0-20W ");
if (scale==2) lcd.print("0-125W");
if (Function==HIGH) //mag loop mode
  {
    lcd.setCursor(0, 3);
    if (band==0) lcd.print("40M"); 
    if (band==1) lcd.print("30M"); 
    if (band==2) lcd.print("20M");
    update_band_info();
  }
return;
}
//******************

void update_band_info()
{
// This routine displays info on whether a band has been set up or not, or it is in progress.
// Is used as part of the main screen display.

if (controller_type==dual_stepper)
  {
  lcd.setCursor(3, 3);  // set cursor to start printing aftrer the band   
  if (band_cap_setup==HIGH)
    {
    lcd.print(" setting ");  
    }
  else
    {
    if(band_cap_settings[band]==-1)
      {
      lcd.print(" Not Set ");
      }
    else 
      {
      lcd.print("         ");  // clear any pervious text
      }
    }
  }
   
return;
}  
//******************

void set_up_band_capactor_manual_adj_screen()
{
//set_up_band_capactor_manual_adj_screen

//set display mode to band cap manual adj
//this is simular to the main screen except the bar graph is replaced with "Band cap adj"
display_mode= band_cap_manual_adj;

lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Power");
lcd.setCursor(7, 0);
lcd.print("Peak");
lcd.setCursor(12, 0);
lcd.print("VSWR");
lcd.setCursor(0, 2);
lcd.print("Band cap adj");
lcd.setCursor(0, 3);
if (band==0) lcd.print("40M"); 
if (band==1) lcd.print("30M"); 
if (band==2) lcd.print("20M");
update_band_info();
return;
}
//******************
void set_up_mag_loop_settings_screen ()
{         
//set display mode to settings screen
display_mode= settings_menu_1; 
    
//set up mag loop settings screen   
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Settings mode");
lcd.setCursor(1, 1);
lcd.print("Band");
lcd.setCursor(1, 2);
lcd.print("Scale");
lcd.setCursor(1, 3);
lcd.print("Light");
lcd.setCursor(9, 1);
lcd.print("Type");
lcd.setCursor(9, 2);
lcd.print("More");
lcd.setCursor(9, 3);
lcd.print("Exit");
lcd.setCursor(0, 1);
lcd.print(">");
settings_cursor=0;
cursor_max=6; //five entries
return;
}
//******************

void set_up_power_meter_settings_screen ()
{ 
//set display mode to settings screen
display_mode= settings_menu_1; 

//set up power meter settings screen
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Settings mode");
lcd.setCursor(1, 1);
lcd.print("Light");
lcd.setCursor(1, 2);
lcd.print("Inputs");
lcd.setCursor(1, 3);
lcd.print("Restore");
lcd.setCursor(9, 1);
lcd.print("Exit");
lcd.setCursor(0, 1);
lcd.print(">");
settings_cursor=0;
cursor_max=4; //four entries
return;
}
//******************

void set_up_band_screen()
{
//set display mode to band select
display_mode= settings_band;

//set up  screen

lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Select Band");
lcd.setCursor(1, 1);
lcd.print("40M");
if(controller_type==dual_stepper && band_cap_settings[0]==-1)lcd.print(" Not Setup"); //display bans capacitor set up info if in dual stepper mode
lcd.setCursor(1, 2);
lcd.print("30M");
if(controller_type==dual_stepper && band_cap_settings[1]==-1)lcd.print(" Not Setup"); //display bans capacitor set up info if in dual stepper mode
lcd.setCursor(1, 3);
lcd.print("20M");
if(controller_type==dual_stepper && band_cap_settings[2]==-1)lcd.print(" Not Setup"); //display bans capacitor set up info if in dual stepper mode
//set cursor to current band value
settings_cursor=band;
//place cursor on screen at current band value
lcd.setCursor(settings_band_cursor_array[settings_cursor*2], settings_band_cursor_array[(settings_cursor*2)+1]);
lcd.print(">");
cursor_max=3; //three entries
return;  
}
//******************

void set_up_scale_screen()
{
//set up scale select screen

//set display mode to scale select
display_mode= settings_scale;
          
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Select Scale");
lcd.setCursor(1, 1);
lcd.print("0-5W");
lcd.setCursor(1, 2);
lcd.print("0-20W");
lcd.setCursor(1, 3);
lcd.print("0-125W");
cursor_max=3; //three entries 

//set cursor to  current scale setting
settings_cursor=scale;
//place cursor on screen at current scale value
lcd.setCursor(settings_scale_cursor_array[settings_cursor*2], settings_scale_cursor_array[(settings_cursor*2)+1]);
lcd.print(">");
return;   
}
//******************

void set_up_backlight_screen()
{
//set up backlight screen

//set display mode to backlight select
display_mode= settings_backlight;

lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Back Light");
lcd.setCursor(1, 1);
lcd.print("Off");
lcd.setCursor(1, 2);
lcd.print("On");
//set cursor to  current backlight value
settings_cursor=Backlight;
//place cursor on screen at current backlight value 
lcd.setCursor(settings_backlight_cursor_array[settings_cursor*2], settings_backlight_cursor_array[(settings_cursor*2)+1]);
lcd.print(">");
cursor_max=2; //two entries
return;  
}
//******************

void set_up_controller_type_screen()
{
//set up controller screen

//set display mode to controller type select
display_mode= settings_controller_type;

lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Controller type");
lcd.setCursor(1, 1);
lcd.print("Band relay type");
lcd.setCursor(1, 2);
lcd.print("Dual stepper");
//set cursor to  current controller type value
settings_cursor=controller_type;
//place cursor on screen at current backlight value 
lcd.setCursor(settings_controller_type_cursor_array[settings_cursor*2], settings_controller_type_cursor_array[(settings_cursor*2)+1]);
lcd.print(">");
cursor_max=2; //two entries
return;  
}


//******************
void set_up_mag_loop_settings_screen_2 ()
{         
//set display mode to settings screen 2
display_mode= settings_menu_2; 
    
//set up mag loop settings screen 2  
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Settings mode 2");
lcd.setCursor(1, 1);
if (controller_type==dual_stepper) lcd.print("Adj band cap");
if (controller_type==relay_type) lcd.print("Not Available");
lcd.setCursor(1, 2);
lcd.print("See Inputs");
lcd.setCursor(1, 3);
lcd.print("Restore");
lcd.setCursor(11, 3);
lcd.print("Exit");
lcd.setCursor(0, 1);
lcd.print(">");
settings_cursor=0;
cursor_max=4; //four entries
return;
}
//******************

void set_up_inputs_display_screen()
{
//set up inputs screen 
//set display mode to inputs screen
display_mode= inputs_mode;

if (Function==HIGH)

  {
  //in mag loop mode, also display End Point detect reading
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Inputs");  
  lcd.setCursor(0, 1);
  lcd.print("EPD:  ");
  lcd.setCursor(0, 2);
  lcd.print("Vfor: "); 
  lcd.setCursor(0, 3);
  lcd.print("Vrev: ");
  }
else
  {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Inputs");  
  lcd.setCursor(0, 1);
  lcd.print("Press to exit");
  lcd.setCursor(0, 2);
  lcd.print("Vfor: "); 
  lcd.setCursor(0, 3);
  lcd.print("Vrev: ");
  } 
return;
}
//******************

void display_values()
{
// this routine displays the instantious values of the inputs into the display inputs screen
lcd.setCursor(6, 2);
// insert leading zeros
if (Vfor<1000) lcd.print("0");
if (Vfor<100) lcd.print("0");
if (Vfor<10) lcd.print("0");
lcd.print(Vfor);
lcd.setCursor(6, 3);
// insert leading zeros
if (Vrev<1000) lcd.print("0");
if (Vrev<100) lcd.print("0");
if (Vrev<10) lcd.print("0");
lcd.print(Vrev);
if (Function==HIGH)
  {
  // Display Magloop controller EPD value
  lcd.setCursor(6, 1);
  // insert leading zeros
  if (Vepd<1000) lcd.print("0");
  if (Vepd<100) lcd.print("0");
  if (Vepd<10) lcd.print("0");
  lcd.print(Vepd);
  }
return;
}
//******************

void set_up_configuration_mode_screen ()
{
//set display mode to configuration screen
display_mode= configuration_mode;

//set up configuration screen
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Conf mode");
lcd.setCursor(1, 1);
lcd.print("Main to start");
lcd.setCursor(1, 2);
lcd.print("Band to start");
lcd.setCursor(1, 3);
lcd.print("Show band vals");
lcd.setCursor(0, 1);
lcd.print(">");
settings_cursor=0;
cursor_max=3; //three entries 
return;
}
//******************



void set_up_defaults_restored_screen()
{
//set display mode to to default restored
display_mode= default_restored;
display_timer=3000; //display message for 3 seconds
lcd.clear();
lcd.setCursor(0, 1);
lcd.print("Restored.");
return; 
}
//******************

void display_bargraph()
{
//is routine diaplays the power bargraph in the main screen
// there are 20 bargraph positions
//for 0-5W multiply  power by 4 to give reading
//for 0-20W use value as is
//for 0-125 divide by 6.25
int bar_value;
int peak_bar_value;
float temp_power;
float temp_peak_power;
switch (scale)
  {
  case 0:
    //0-5W scale
    temp_power= Power*4;
    temp_peak_power=PK_Power*4;
    break;

  case 1:
    //0-20W scale
    temp_power= Power;
    temp_peak_power=PK_Power;
    break;
    
  case 2:
    //0-125W scale
    temp_power= Power/6.25;
    temp_peak_power=PK_Power/6.25;
    break;
  }
if (temp_power >= 20) 
      {
      bar_value = 20; //set to maximum value
      }
    else
      {
      bar_value = (int)temp_power;
      if (temp_power-(int)temp_power>=0.5)bar_value=bar_value+1; //round up if => 0.5
      }
 if (temp_peak_power >= 20) 
      {
      peak_bar_value = 20; //set to maximum value
      }
    else
      {
      peak_bar_value = (int)temp_peak_power;
      if (temp_peak_power-(int)temp_peak_power>=0.5)peak_bar_value=peak_bar_value+1;
      }     


if( bar_value==0 & power_present==HIGH) bar_value=1;  //if power present flag set then set first bar to show something


//The bar graph is t 20 bars made up from 10 chracters spaces on the display, so each characher on the display shows one of the 
//following five symbols:-
//
//code_0 two dots - no power 
//code_1 power bar dot 
//code_2 2 power bars
//code_3 peak bar dot
//code_4 power bar peak bar
//code_5 dot peak bar

lcd.setCursor(0, 2);
int x;

//First display power in all the characters up to and including the last even number of bar_value.
//Also as Peak bar value is always equal to or greater than bar value, see if the last even number of power bar
//is the same as the peak value. if so display the peak bar  in the last character. 
for (x=1; x<= bar_value/2;x=x+1)
  {
  if (bar_value/2==x & peak_bar_value==bar_value & peak_bar_value%2==0)
    {
     lcd.write(byte(4)); // power bar peak bar
    }
  
 else
    {
    lcd.write(byte(2)); // write double power bars 
    }
  }

  //now deal with an odd number power bar if it exists
  
if ((bar_value %2)!=0) //get remainder when dividing bar value by 2. If 0 then all double bars so nothing to do.
  {
  //as a minumum need to display a power bar and a dot in this character position.
  //see if the peak power is equal to  power bar or adjacent to it
  if ((peak_bar_value/2)+(peak_bar_value%2)==x) 
    {
     //peak needs to be displayed in this character
     if (peak_bar_value%2==1) //ie its coincident with the power bar value
      {
       lcd.write(byte(3)); // write peak bar dot
       x=x+1; 
      }
     else
     {
      lcd.write(byte(4)); // write power bar peak bar
      x=x+1; 
     }
    }
  else
    {
    lcd.write(byte(1)); // write power bar and dot
    x=x+1;
    }
  }

//now write in double dots to the end and add in peak if it hasnt alread been added.
  
while (x<=10) //write double dots in the remaining positions
  {
    if ((peak_bar_value/2)+(peak_bar_value%2)==x) 
    {
     //peak needs to be displayed in this character
     if (peak_bar_value%2==1) //ie peaks is in  the first position on the 
      {
       lcd.write(byte(3)); // write peak bar dot
       x=x+1; 
      }
     else
     {
      lcd.write(byte(5)); // write dot peak bar
      x=x+1; 
     }
    }
  else
    {
    lcd.write(byte(0)); // write double dot
    x=x+1;
    }
      
  }  
return;  
}
//******************

void display_power_and_SWR()
{
//this routine displays the calaculated power and VSWR readings on the main screen
  //display power
lcd.setCursor(0, 1);
if(Power<9.9) lcd.print("  "); //remove leading zeros
if(Power>=9.9 & Power<100) lcd.print(" ");
lcd.print(Power,1); //print power to one decimal place


//display peak power
lcd.setCursor(6, 1);
if (PK_Power_timer==0) PK_Power=Power;// Peak hold time expired then set Pk Power to current power
if(PK_Power<9.9) lcd.print("  "); //remove leading zeros
if(PK_Power>=9.9 & PK_Power<100) lcd.print(" ");
lcd.print(PK_Power,1); //print PK power to one decimal place

//display SWR
lcd.setCursor(12, 1);
if (SWR==0) // SWR will be set to zero if there is no power to clear value
  {
    lcd.print("    ");
  }
  else
  {
    if (SWR <10)
      {
      lcd.print(SWR,2); //print SWR to Two decimal places
      }
    else
      {
      if (SWR<100)
        {
        lcd.print(SWR,1); //print to one decimal place 
        }
      else
        {
        lcd.print("V HI"); 
        }
      }
  }
return;
}
//******************

void display_power_too_high_warning()
{
//stay in this routine till power levels have been reduced
lcd.clear();
lcd.setCursor(0, 2);
lcd.print("Power too high !");
while (power_warning==HIGH)
  {
    get_bridge_inputs ();
  }
//Power level restored to acceptable level

//clear flags in case pressed during waring
clear_flags ();

// return to main screen
set_up_main_screen();

return;
}
//******************

void display_endpoint_voltage_too_high_warning()
{
//stay in this routine till end point voltage levels have been reduced
lcd.clear();
lcd.setCursor(0, 1);
lcd.print("End point dect");
lcd.setCursor(0, 2);
lcd.print("Voltage too HIGH");
while (end_point_warning==HIGH)
  {
    read_end_point_detectors();
  }
//EPD voltage restored to acceptable level

//clear flags in case pressed during waring
clear_flags ();
// return to main screen
set_up_main_screen();

return;
}
//******************

void display_inputs_reversed_warning()
{
//stay in this routine till have been reversed or power removed
lcd.clear();
lcd.setCursor(0, 1);
lcd.print("RF Connections");
lcd.setCursor(0, 2);
lcd.print("reversed !"); 
while (power_reversed==HIGH)
  {
    get_bridge_inputs ();
  }
//Connections restored 

 //clear flags in case pressed during waring
clear_flags ();

// return to main screen
set_up_main_screen();

return;
}
//******************


void display_band_capacitor_values ()
{
//Display the band capacitor values

display_mode= show_band_cap_values;
display_timer=3000; //display message for 3 seconds
lcd.clear();  
lcd.setCursor(0, 0);
lcd.print("Band Cap Values");
lcd.setCursor(0, 1);
lcd.print("40M ");
lcd.print(band_cap_settings[0]); 
lcd.setCursor(0, 2);
lcd.print("30M ");
lcd.print(band_cap_settings[1]);
lcd.setCursor(0, 3);
lcd.print("20M ");
lcd.print(band_cap_settings[2]);
return;
}
//******************

void setup_band_capacitor_tuning_screen ()
{
display_mode= band_capacitor_tuning_screen;
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Set up band Cap");
lcd.setCursor(0, 1);
lcd.print("Band: ");
if (band==0) lcd.print("40M. Set"); 
if (band==1) lcd.print("30M. Set"); 
if (band==2) lcd.print("20M. Set");
lcd.setCursor(0, 2);
if (band==0) lcd.print("TX to 7.100MHz");
if (band==1) lcd.print("TX to 10.125MHz");
if (band==2) lcd.print("TX to 14.175MHz");
lcd.setCursor(0, 3);
lcd.print("Please wait");
return;
}
//******************


void display_band_cap_to_start_epd_error ()
{
display_mode= band_cap_to_start_epd_error;
display_timer=3000; //display message for 3 seconds  
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Band Capacitor");
lcd.setCursor(0, 1);
lcd.print("end point not"); 
lcd.setCursor(0, 2);
lcd.print("reached. Please");
lcd.setCursor(0, 3);
lcd.print("check.");
return;
}

//******************
void display_main_cap_to_start_epd_error ()
{
display_mode= main_cap_to_start_epd_error;
display_timer=3000; //display message for 3 seconds 
lcd.clear();
lcd.setCursor(0, 0);
lcd.print("Tuning Capacitor");
lcd.setCursor(0, 1);
lcd.print("end point not"); 
lcd.setCursor(0, 2);
lcd.print("reached. Please");
lcd.setCursor(0, 3);
lcd.print("check.");
return;
}



//***********************************
//
//General routines
//
//***********************************

void clear_flags ()
{
//clear the rotary encoder, buttons and display timer flags
single_press=LOW; 
double_press=LOW; 
encoder_changed_state=LOW;
long_press=LOW;
display_timer_expired=LOW; 
return;
}
//******************



void get_bridge_inputs ()
{
  //read the brigde inputs 
int temp_vfor;
int temp_vrev;
//get forward voltage
temp_vfor=analogRead(vforward);
//store in rolling agerage array
vforward_average_array[rolling_average_index]=temp_vfor;

// get reverse voltage
temp_vrev=analogRead(vreverse);
//store in rolling agerage array
vreverse_average_array[rolling_average_index]=temp_vrev;

//increnment rolling average pointer and roll one at end
rolling_average_index=rolling_average_index+1;
if ( rolling_average_index>=20) rolling_average_index=0;

//calculate rolling average
Vfor =0;
Vrev =0;
for (int x=0;x<20;x=x+1)
  {
   Vfor=Vfor+vforward_average_array[x];
   Vrev=Vrev+vreverse_average_array[x];
  }
Vfor=Vfor/20;
Vrev=Vrev/20;
  
//see if power present
if (temp_vfor >minimum_power || temp_vrev>minimum_power)
  {
  //power is present
  power_present=HIGH;

  //Check if power level is too high or inputs have been reversed
  power_reversed=LOW;
  power_warning=LOW;
  //see if power level too high
  if (temp_vrev >maximum_power || temp_vrev>maximum_power) power_warning=HIGH;
  //see if input reversed
  if (temp_vrev>=(temp_vfor+10)) power_reversed=HIGH;  //allow for any DC offset on the inputs.
  
  }
else
  {
   //Power not present
   power_present=LOW;  
  }

return; 
}
//******************

void calculate_power_and_SWR()
{
//0-125 watt model
//power in = vsence squared *0.7071^2 to get to rms *n^2/50  where n is the number of turns on sence transformer 
//number of turns =25

int n=25; 

if (power_present==HIGH) //check power present to avoid a devide by zero error
  {
  Power= sq(Vfor*V_ref/1023)*0.5*n*n/50;
  if(Power>PK_Power)
    {
      PK_Power=Power;
      PK_Power_timer=3000;
    }

   SWR=((1+(float(Vrev)/float(Vfor)))/(1-(float(Vrev)/float(Vfor))));
  if(SWR<1) SWR=0;  //it is sometimes possible to during initial power up to see Vrev >Vfor  giving a negative results
  }
else
  {
  Power=0;
  SWR=0;  
  }
return;
}
//******************

void increment_setting_pointer( int ArrayName[])
{
//this routine increments the settings pointer up to the max and then rolls it back to zero. 
//it then displays the cursor based upon the cursor value and in a position defined in the array passed
//clear cursor at current position
lcd.setCursor(ArrayName[settings_cursor*2], ArrayName[(settings_cursor*2)+1]);
lcd.print(" ");
settings_cursor=settings_cursor+1;
if (settings_cursor==cursor_max) settings_cursor=0;
//display cursor at new position
lcd.setCursor(ArrayName[settings_cursor*2], ArrayName[(settings_cursor*2)+1]);
lcd.print(">");  
return; 
}
//******************
void decrement_setting_pointer( int ArrayName[])
{
//this routine decrements the settings pointer up to the minimum and then rolls it. 
//it then displays the cursor based upon the cursor value and in a position defined in the array passed
//clear cursor at current position
lcd.setCursor(ArrayName[settings_cursor*2], ArrayName[(settings_cursor*2)+1]);
lcd.print(" ");
settings_cursor=settings_cursor-1;
if (settings_cursor==-1) settings_cursor=cursor_max-1;
//display cursor at new position
lcd.setCursor(ArrayName[settings_cursor*2], ArrayName[(settings_cursor*2)+1]);
lcd.print(">");
return; 
}
//******************     

void set_ee_to_default_values()
{
//used to restore the working and stored values of the band, scale backlight and band capacitor values to the default start values.
EEPROM.update(0, 0x55);
EEPROM.update(1, 0xAA);
band=0;  //set to 40M
EEPROM.update(2, band);
scale=2; //set to 0-125w
EEPROM.update(3, scale);
//turn on backlight
Backlight=1; 
EEPROM.update(4, Backlight);
digitalWrite(back_light, HIGH); 
//set band capacitor values back to not set
band_cap_settings[0]=-1;
band_cap_settings[1]=-1;
band_cap_settings[2]=-1;
EEPROM.put(5,band_cap_settings[0]); //as value is an integer it take 2 byte to store info
EEPROM.put(7,band_cap_settings[1]);
EEPROM.put(9,band_cap_settings[2]);
controller_type=relay_type; //relay band switched type 
EEPROM.update(11, controller_type);

return;
}
//******************

void update_ee()
{
//used to upodate the EEprom with the latest values. Only those that have changed get altered.
EEPROM.update(0, 0x55);
EEPROM.update(1, 0xAA);
EEPROM.update(2, band);
EEPROM.update(3, scale);
EEPROM.update(4, Backlight);
EEPROM.put(5,band_cap_settings[0]); //as value is an integer it take 2 byte to store info
EEPROM.put(7,band_cap_settings[1]);
EEPROM.put(9,band_cap_settings[2]);
EEPROM.update(11, controller_type);
return;  
}
//******************

void clear_average_arrays ()
{
for (int x=0;x<20;x=x+1)
  {
  vforward_average_array[x]=0;
  vreverse_average_array[x]=0;
  }  
rolling_average_index=0;
 
return;   
}
//******************

void stepper_off()
{
//Turn off the stepper motor output
digitalWrite(Stepper_1, LOW);
digitalWrite(Stepper_2, LOW);
digitalWrite(Stepper_3, LOW);
digitalWrite(Stepper_4, LOW);
stepper_powered =LOW;
return; 
}

//******************

void opperate_stepper()
{

switch (stepper_phase)
      {
       case 0:
          digitalWrite(Stepper_1, HIGH);
          digitalWrite(Stepper_2, HIGH);
          digitalWrite(Stepper_3, LOW);
          digitalWrite(Stepper_4, LOW);
          break;
       case 1:
          digitalWrite(Stepper_1, LOW);
          digitalWrite(Stepper_2, HIGH);
          digitalWrite(Stepper_3, HIGH);
          digitalWrite(Stepper_4, LOW);
          break;
       case 2:
          digitalWrite(Stepper_1, LOW);
          digitalWrite(Stepper_2, LOW);
          digitalWrite(Stepper_3, HIGH);
          digitalWrite(Stepper_4, HIGH);
          break;
       case 3:
          digitalWrite(Stepper_1, HIGH);
          digitalWrite(Stepper_2, LOW);
          digitalWrite(Stepper_3, LOW);
          digitalWrite(Stepper_4, HIGH);
          break; 
       default:
          digitalWrite(Stepper_1, HIGH);
          digitalWrite(Stepper_2, HIGH);
          digitalWrite(Stepper_3, LOW);
          digitalWrite(Stepper_4, LOW);      
          stepper_phase=0;
         break; 
      }


//increment or decrement stepper phase for next opperation      
if (stepper_direction==LOW)
    {
     stepper_phase=stepper_phase +1;
     if (stepper_phase==4) stepper_phase=0;         
    }
  else
    {
    stepper_phase=stepper_phase -1;
     if (stepper_phase==-1) stepper_phase=3;
    }

stepper_powered =HIGH;  //indicate the stepper is power up
stepper_on_timer=stepper_on_time; //reset stepper power on timer

//set the stepper speed ie indicate when this routine can be called again
if (stepper_speed==HIGH)
  {
  step_interval_timer=step_interval_time;
  }
else
  {
  step_interval_timer=step_interval_time_slow;
  }

return;        
}
//******************


void stepper_deslack()
{
//this routine opperates the stepper a number of times when the direction has changed to take up the slack in the stepper gear drive.
for( int x=1;x<=deslack_count; x=x+1)
  {
  while (step_interval_timer>0)
    {
    //wait  
    }
  opperate_stepper(); 
  }      
return; 
}
//******************

void tune()
{
//This routine is used to tune the main or band capacitor as required.
if (tuning_in_progress==LOW)
  {
  if (controller_type==dual_stepper && band_cap_settings[band]==-1) //see if controller is dual stepper type and if band capacitor has a setting, if not this needs to be set up. 
    {
    //Band capactor has not been set up. Try to set it up
    setup_band_capacitor_tuning_screen (); 

    if (set_tuning_cap_to_start()==1)
      {
      //there was a problem, display message
      display_main_cap_to_start_epd_error ();             
      }
  
    else
      {
      set_tuning_cap_to_centre();
    
      if (set_band_cap_to_start()==1)
        {
        //there was a problem, display message 
        display_band_cap_to_start_epd_error ();
        }
      else
        {
        //Still in band capacitor tuning screen. Change the "please wait" to "Turn on TX now". 
        lcd.setCursor(0, 3);
        lcd.print("Turn TX on now  ");
        display_timer=3000; //display message for 3 seconds to allow message to be seen
        point_stepper_to_band_tuning_capacitor();      
        band_cap_setup=HIGH;  //used to display "setting next to the band"
        tuning_in_progress=HIGH;
        wait_for_power_timer=wait_for_power_time;
        step_counter=0;        
        }
      }
    }
  else
    {
    //band capacitor has a setting 
    //tune the main capacitor
    tuning_reverse_check=LOW;
    lcd.setCursor(12, 3);
    lcd.print("Tune");
    if (controller_type==dual_stepper) point_stepper_to_main_tuning_capacitor(); 
    tuning_in_progress=HIGH;
    wait_for_power_timer=wait_for_power_time;
    step_counter=0;
    }
  }
else
  {
  //Tuning in progress
  if (power_present==LOW )
    {
     if (wait_for_power_timer >0)
      {
      return; // still waiting for power to be turned on
      }
     else
      {
      lcd.setCursor(12, 3);
      lcd.print("NoPW");
      display_timer=tune_message_time; //display message for tune message time
      
      if ( controller_type== dual_stepper) point_stepper_to_main_tuning_capacitor();
      stepper_speed=HIGH; //restore stepper speed to normal
      //Clear tuning, band set up, and power stable flags
      band_cap_setup=LOW;  
      tuning_in_progress=LOW;
      power_stable=LOW;
      // update band info
      update_band_info();
      }  
    }
  else
    {
     // Power is present, is the power stable ?
     if (power_stable==LOW)
       {
       if (power_stable_timer_flag==LOW) 
         { 
         //this is the first time the power has been present set up the stability wait timer 
         wait_for_power_stable_timer = wait_for_power_stable_time;
         power_stable_timer_flag=HIGH; //indicate timer is running
         }
       else
         {
         if (wait_for_power_stable_timer==0) //wait for the timer to expire and then set the stability flag
           {
           power_stable_timer_flag=LOW;
           power_stable=HIGH; 
           starting_swr=SWR; //save current swr to see if when tuning swr if getting better or worse
           //see if still in band capacitor set up screen showing "please turn on TX" 
           if (display_mode==band_capacitor_tuning_screen )
              {
              if (display_timer>2) //if timer has more that 2ms to run, stop it and change  to main screen, otherwise let the display timer do the job.
                {
                display_timer=0;
                set_up_main_screen();
                lcd.setCursor(12, 3);
                lcd.print("Tune");
                update_band_info();
                }
              }
            }
          }
       }
     else
       {
       //power is stable       
       if (SWR<10) stepper_speed=LOW; //set it to slow speed to allow more time for the stepper to move
       if (SWR<=3) //power present and stable.Is SWR less than 3?  
         {
         lcd.setCursor(12, 3);
         lcd.print("OK  ");
         display_timer=tune_message_time; //display message for tune message time
         if ( controller_type==dual_stepper) point_stepper_to_main_tuning_capacitor();
         stepper_speed=HIGH; //restore stepper speed to normal
         tuning_in_progress=LOW;
         power_stable=LOW; 
         if (band_cap_setup==HIGH)       
           {
           //save band capacitor stepper settings
           band_cap_settings[band]=step_counter; //store setting in array
           update_ee(); //store new band cap setting in ee
           band_cap_setup=LOW;
           }
         update_band_info();
         }
       else
         {
         //power present but SWR > 3 continue tuning
         if (band_cap_setup==LOW)
           {
           //tuning the main capacitor
           //See if the direction of the stepper is making the swr worse, if so reverse the direction. this can only be done once
           if  (tuning_reverse_check==LOW) 
             {
             if (SWR>=starting_swr+1.00)
               {
               //Match is getting worse, reverse tuning direction and set 
               //reverse check flag so direction will not be reversed again this tuning cycle
               //also reset the step counter
          
               if (stepper_direction==LOW)
                 {
                 stepper_direction=HIGH;   
                 }
               else
                 {
                 stepper_direction=LOW; 
                 }
               tuning_reverse_check=HIGH;
               step_counter=0;
               }
             } 
           } 
          if (step_interval_timer==0)
           {
           if (step_counter==max_step_count)
             {
             lcd.setCursor(12, 3);
             lcd.print("NOK ");
             display_timer=tune_message_time; //display message for tune message time
             if (controller_type==dual_stepper) point_stepper_to_main_tuning_capacitor();
             stepper_speed=HIGH; //restore stepper speed to normal
             //Clear tuning, band set up, and power stable flags
             band_cap_setup=LOW;
             tuning_in_progress=LOW;
             power_stable=LOW;
             // update band info
             update_band_info(); 
             }
           else
             {
             //opperate stepper in its current direction
             opperate_stepper();
             step_counter=step_counter+1;  
             }
           }
         }
       }
    }
  } 
return;     
}  
//******************

void set_up_band_relays()
{
//set up the band relay according to band setting
switch (band)
{
 case 0:  //40M relay 1 opperated relay 2 released
  digitalWrite(Relay_1, HIGH); 
  digitalWrite(Relay_2, LOW); 
  break;
 case 1: //30M relay 1 released relay 2 opperated
  digitalWrite(Relay_1, LOW); 
  digitalWrite(Relay_2, HIGH); 
  break;
 case 2: //20M relay 1 released relay 2 released
  digitalWrite(Relay_1, LOW); 
  digitalWrite(Relay_2, LOW); 
  break;  
}
return;  
}
//******************


int set_tuning_cap_to_start()
{
//this routine sets the main tuning capacitor to the start postion. 
point_stepper_to_main_tuning_capacitor();

//operate stepper to drive it back to the home end point.
stepper_direction=main_tuning_cap_minimum_direction; //set it to minimum capactance
step_counter=0;

while (step_counter<max_step_count && end_point_detector_one == LOW) //exit if step count = max ie stepper has rotated 180 deg or once the end point detect has opperated. 
  {
  read_end_point_detectors();
  if (step_interval_timer==0)
    {
    opperate_stepper();
    step_counter=step_counter+1;
    }
  }
  
if (step_counter!= max_step_count) 
  {
  //now step forward till the end point detector goes off. This will ensure any slack in the stepper has been removed.
  stepper_direction=main_tuning_cap_increase_C_direction; //set it to increase C
  step_counter=0;
    
  while (step_counter<max_step_count && end_point_detector_one == HIGH) //exit if step count = max ie stepper has rotated 180 deg or once the end point detect has been released. 
    {
    read_end_point_detectors();
    if (step_interval_timer==0)
      {
      opperate_stepper();
      step_counter=step_counter+1;
      }
    } 
   
  return(0);
   }
//there was a problem, the main tuning capacitor epd did not stop the stepper
point_stepper_to_main_tuning_capacitor();
return(1); 
}
//******************

void set_tuning_cap_to_centre()
{
//this routine sets the main tuning capacitor to the center postion. It is used after the capactor has been set to start postion.
point_stepper_to_main_tuning_capacitor();
// now step on to position capacitor to half way between max and min capacitance 
step_counter=0;
while (step_counter<mid_c_position ) //exit when step count = mid_c_position  ie stepper has rotated 45 deg. 
   {
   if (step_interval_timer==0)
     {
     opperate_stepper();
     step_counter=step_counter+1;
     }
    }  
return;
}

//******************

int set_band_cap_to_start()
{
//this routine drives the band capacitor to the start position
//set stepper relay to allow band capacitor to be adjusted
point_stepper_to_band_tuning_capacitor();

//operate stepper to drive it back to the home end point.
stepper_direction=band_cap_minimum_direction; //set it to minimum capactance
step_counter=0;
while (step_counter<max_step_count && end_point_detector_two == LOW) //exit if step count = max ie stepper has rotated 180 deg or once the end point detect has opperated. 
  {
  read_end_point_detectors();
  if (step_interval_timer==0)
    {
    opperate_stepper();
    step_counter=step_counter+1;
    }
  }
if (step_counter!= max_step_count) 
  {
   //now step forward till the end point detector goes off. This will ensure any slack in the stepper has been removed.
   stepper_direction=band_cap_increase_C_direction; //set it to increase C
   step_counter=0;

   while (step_counter<max_step_count && end_point_detector_two == HIGH) //exit if step count = max ie stepper has rotated 180 deg or once the end point detect has been released. 
     {
     read_end_point_detectors();
     if (step_interval_timer==0)
        {
        opperate_stepper();
        step_counter=step_counter+1;
        }
     }
  if (step_counter!= max_step_count)
    {
    return(0);  
    }
  }
//there was a problem, the band capacitor epd did not stop the stepper
point_stepper_to_main_tuning_capacitor();
return(1);  
}
//******************

void point_stepper_to_main_tuning_capacitor()
{
while (step_interval_timer>0)
  {
   //wait for stepper to complete last step, till its timer zero 
  }

stepper_off();  //turn stepper outputs off   
digitalWrite(Relay_1, LOW); 
return;
}
//******************

void point_stepper_to_band_tuning_capacitor()
{
while (step_interval_timer>0)
  {
   //wait for stepper to complete last step, till its timer zero 
  }
  
stepper_off();  //turn stepper outputs off 
digitalWrite(Relay_1, HIGH); 
return;
}
//******************


void set_band_cap_to_value()
{
//this routine sets the band capacitor to the previously stored setting
//first set the capacitor to the start position   
if (set_band_cap_to_start()==1) // get band capacitor to start position and check if there was an issue with the epd
  {
  //there was a problem, message has been displayed, return to main screen
  set_up_main_screen();                  
  }
else
  {
  // all ok now step on to band value
  step_counter=0;
  stepper_direction=band_cap_increase_C_direction; //set it to increase C
  while (step_counter<band_cap_settings[band] ) //exit when step count = band count value   
    {
    if (step_interval_timer==0)
      {
      opperate_stepper();
      step_counter=step_counter+1;
      }
    }
  }
point_stepper_to_main_tuning_capacitor(); // restore tuning control to the main tuning capacitor 
return;
}
//******************

void read_end_point_detectors()
{
//This routine reads the end point dectector analogue input and determines which if any or both are active

Vepd=analogRead(end_point_detect); //Measure end_point_detect voltage
if (Vepd >= epd_max)
  {
    end_point_warning=HIGH;
  }
else
  {
   end_point_warning=LOW; 
  } 
// determing which if any end point detectors are opperated

end_point_detector_one = LOW;
end_point_detector_two = LOW;

if ( Vepd>= epd_1_min && Vepd <= epd_1_max)
   {
   end_point_detector_one = HIGH;
   end_point_detector_two = LOW;
   }
else
   {
   if ( Vepd>= epd_2_min && Vepd <= epd_2_max)
      {
      end_point_detector_one = LOW;
      end_point_detector_two = HIGH;
      }
   else
      {
      if ( Vepd>= epd_1and2_min && Vepd <= epd_1and2_max)
         {
         end_point_detector_one = HIGH;
         end_point_detector_two = HIGH;
         }

      }
   }
return; 
}
//******************

//******************************************
// Interupt service routine
//******************************************
SIGNAL(TIMER0_COMPA_vect) 
{

//service timers
if(encoder_clock_timer>0) encoder_clock_timer=encoder_clock_timer-1;
if(encoder_switch_timer>0) encoder_switch_timer=encoder_switch_timer-1;
if(encoder_switch_two_second_timer>0) encoder_switch_two_second_timer=encoder_switch_two_second_timer-1;
if(encoder_switch_half_sec_timer>0) encoder_switch_half_sec_timer=encoder_switch_half_sec_timer-1;
if(PK_Power_timer>0) PK_Power_timer=PK_Power_timer-1;
if(stepper_on_timer>0) stepper_on_timer=stepper_on_timer-1;
if(step_interval_timer>0) step_interval_timer=step_interval_timer-1;
if(wait_for_power_timer>0) wait_for_power_timer=wait_for_power_timer-1;
if(display_timer==1) display_timer_expired=HIGH; // if display timer is 1 then it will get set to zero on the next line, set time expired flag.
if(display_timer>0) display_timer=display_timer-1;
if(wait_for_power_stable_timer>0) wait_for_power_stable_timer=wait_for_power_stable_timer-1;



//scan rotary encoder
if (digitalRead(encoder_clock)==LOW)  //LOW = operated HIGH = released
  {
  if (encoder_clock_last_stable_state==HIGH)  //has changed state
    {
    if(encoder_clock_debounce_flag==LOW)
      {
      encoder_clock_timer=enc_debounce_time;
      encoder_clock_debounce_flag=HIGH;
      }
      else
      {
      if(encoder_clock_timer==0)
        {
        encoder_clock_last_stable_state=LOW;
        encoder_clock_debounce_flag=LOW;
        encoder_changed_state=HIGH;
        encoder_direction=LOW;
        if (digitalRead(encoder_data)==HIGH) encoder_direction=HIGH;
        }
    
      }
    } 
    else //no change of state 
    {
    encoder_clock_debounce_flag=LOW;
    }
  }
  else
  {
  if (encoder_clock_last_stable_state==LOW)  //has changed state
    {
    if(encoder_clock_debounce_flag==LOW)
      {
      encoder_clock_timer=enc_debounce_time;
      encoder_clock_debounce_flag=HIGH;
      }
     else
      {
      if(encoder_clock_timer==0)
        {
        encoder_clock_last_stable_state=HIGH;
        encoder_clock_debounce_flag=LOW;
 
        }
    
      }
    }
    else //no change of state
    {
    encoder_clock_debounce_flag=LOW;
    } 
  }



//ISR encoder_switch


//note the digital pin = low for oppeated. The rest of the route assumes the logic of HIGH = opperated 
if (digitalRead(encoder_switch_pin)==LOW)  //LOW = pressed HIGH = released  
  {
  //button pressed
  
  if (encoder_switch_last_stable_state==LOW)  //has changed state
    {
    if(encoder_switch_debounce_flag==LOW)
      {
      encoder_switch_timer=enc_switch_debounce_time;
      encoder_switch_debounce_flag=HIGH;
      }
      else
      {
      if(encoder_switch_timer==0)
        {
        encoder_switch_last_stable_state=HIGH;
        encoder_switch_debounce_flag=LOW;
        encoder_switch_two_second_timer=2000;
        encoder_switch_button_held_flag=LOW;
        }
    
      }
    } 
    else 
    {
    if(encoder_switch_two_second_timer==0)
      {
      if(encoder_switch_button_held_flag==LOW)
        {
        encoder_switch_button_held_flag=HIGH;
        long_press=HIGH;
        }
      }
      else
      {
      encoder_switch_debounce_flag=LOW;
      }
    }
  }
  
  else
  {
  //button not pressed
  
  if (encoder_switch_last_stable_state==HIGH)  //has changed state
    {
    if(encoder_switch_debounce_flag==LOW)
      {
      encoder_switch_timer=enc_switch_debounce_time;
      encoder_switch_debounce_flag=HIGH;
      }
      else
      {
      if(encoder_switch_timer==0)
        {
        encoder_switch_two_second_timer=0;
        
        if(encoder_switch_button_held_flag==HIGH)
          {
          encoder_switch_button_held_flag=LOW;
          encoder_switch_last_stable_state=LOW;
          encoder_switch_debounce_flag=LOW;
          }
          else
          {
          if(encoder_switch_half_sec_timer_flag==LOW)
            {
            encoder_switch_half_sec_timer=500;
            encoder_switch_half_sec_timer_flag=HIGH;
            encoder_switch_debounce_flag=LOW;
            encoder_switch_last_stable_state=LOW;
            }
            else
            {
            encoder_switch_half_sec_timer_flag=LOW;
            encoder_switch_debounce_flag=LOW;
            encoder_switch_last_stable_state=LOW;
            double_press=HIGH;  
            }
          }
        }
    
      }
    }
    else 
    {
    if(encoder_switch_half_sec_timer_flag==HIGH)
      {
      if(encoder_switch_half_sec_timer==0)
        {
        encoder_switch_half_sec_timer_flag=LOW;
        single_press=HIGH;
        }
      }

    } 

  }

//end of encoder switch 
//end of ISR
}
